// Copyright 2001, Jonathan Bachrach.  See file TERMS.

import java.io.*;
import java.util.*;
import java.lang.*;

/**
 * Java PreProcessor
 * @author  Jonathan Bachrach
 * @author  Keith Playford
 * @version 0.9, 03OCT00
 */
public class JSE {
    static String suffix = ".jse";
    static FragmentList parse (String string) throws Exception {
	LooseFragmentParser parser
	    = new LooseFragmentParser
		(new StringReader(string), true);
	FragmentList fragments
	    = parser.parse();
	fragments.pprint(Fragment.out); Fragment.out.println();
	Fragment.out.println("LENGTH : " + fragments.size());
	List tokens
	    = fragments.tokens();
	Fragment.out.println(tokens);
	return fragments;
    }

    static int exec (String[] cmd, String[] envp) throws Exception {
	Runtime runtime     = Runtime.getRuntime();
	Process child       = runtime.exec(cmd);
	InputStream cmdOut  = child.getInputStream();
	InputStreamReader r = new InputStreamReader(cmdOut);
	BufferedReader in   = new BufferedReader(r);
	String line;
	while ((line = in.readLine()) != null) 
	    Fragment.out.println(line);
	int result = child.waitFor();
	return result;
    }

    static void expandStreams
	(InputStream input, OutputStream out, 
	 boolean isTraced, boolean isPretty, boolean isRecursive) 
	   throws Exception {
	PrettyStream output = new PrettyStream(new PrintStream(out));
	LooseFragmentParser parser
	    = new LooseFragmentParser(input, isTraced);

	FragmentList fragments
	    = parser.parse();

	/*
	if (isPretty)
	    fragments.pprint(Fragment.out); 
	else
	    fragments.lprint(Fragment.out); 
	Fragment.out.println();
	*/

	MacroExpander expander
	    = new MacroExpander();

	LooseFragmentParser.debugOut("// MARKING MACROS");
	fragments = fragments.markMacros(expander);
	LooseFragmentParser.debugOut("// AFTER MARKING MACROS " + fragments);
	LooseFragmentParser.debugOut("// AFTER MARKING MACROS", fragments);
	LooseFragmentParser.debugOut("// EXPANDING MACROS " + expander.haveMacros());
	if (expander.haveMacros())
	    fragments.expandMacros(expander, isRecursive);
	fragments.expandTemplates();

	if (isPretty)
	    fragments.pprint(output); 
	else
	    fragments.lprint(output); 

	output.println(); output.close();

    }

    static void expandFile 
	  (String filename, 
	   boolean isTraced, boolean isCompiled, boolean isPretty, boolean isRecursive)
	  throws Exception {
	int flen = filename.length();
	int slen = suffix.length();
	if ((flen > slen) && filename.substring(flen-slen).equals(suffix)) {

	    String baseName       = filename.substring(0, flen-slen);
	    String outputFilename = baseName + ".java";
	    File   outputFile     = new File(outputFilename);
	    OutputStream output   = new FileOutputStream(outputFile);
	    try {
		File file         = new File(filename);
		InputStream input = new FileInputStream(file);
		expandStreams(input, output, isTraced, isPretty, isRecursive);
	    } catch (Exception e) {
		System.err.println("JSE exception: "+e);
		e.printStackTrace();   // so we can get stack trace		
	    }	    

	    String classPath = "CLASSPATH=" + System.getProperty("CLASSPATH");
	    String[] envp    = { classPath };
	    String appname   = "javac";
	    String[] cmd     = { appname, outputFilename };
	    if (isCompiled)
		exec(cmd, envp);
	}
    }

    static public void main(String args[]) throws Exception {
	try {
	    if (args.length > 0) {
		boolean isTraced    = false;
		boolean isCompiled  = false;
		boolean isPretty    = true;
		boolean isStdio     = false;
		boolean isRecursive = true;
		for (int i = 0; i < args.length; i++) {
		    if (args[i].equals("-trace"))
			isTraced = true;
		    else if (args[i].equals("-compile"))
			isCompiled = true;
		    else if (args[i].equals("-pretty"))
			isPretty = true;
		    else if (args[i].equals("-lineup"))
			isPretty = false;
		    else if (args[i].equals("-recursive"))
			isRecursive = true;
		    else if (args[i].equals("-one"))
			isRecursive = false;
		    else if (args[i].equals("-stdio"))
			isStdio = true;
		    else
			expandFile
			    (args[i], isTraced, isCompiled, isPretty, isRecursive);
		}
		if (isStdio)
		    expandStreams
			(System.in, System.out, isTraced, isPretty, isRecursive);
	    } else {
		System.err.println
		    ("Usage: java JSE [-trace] ?filename" + suffix + " ...");
		System.exit(-1);
	    }
	    System.exit(0);
	} catch (Exception e) {
	    System.err.println("Main exception: "+e);
	    e.printStackTrace();   // so we can get stack trace		
	    System.exit(-1);
	}	    
    }
}
