// Copyright 2001, Jonathan Bachrach.  See file TERMS.

/**
 * Fragment for marking macro calls
 * @author  Jonathan Bachrach
 * @author  Keith Playford
 * @version 0.9, 03OCT00
 */
public class MacroCall extends CompoundFragment {
    FragmentList   fragments = null;
    SyntaxExpander macro     = null;
    FragmentList modifiers   = FragmentList.fnil;
    IdentifierFragment id    = null;
    public FragmentList getModifiers () { return modifiers; }
    public FragmentList getIdAndBeyond () { 
	for (FragmentList ptr = fragments; ptr != FragmentList.fnil; ptr = ptr.ftail()) {
	    Fragment x = ptr.fhead();
	    if (x instanceof IdentifierFragment 
		&& id.equals((IdentifierFragment)x))
		return ptr;
	}
	return null; 
    }
    public SyntaxExpander getExpander () { return macro; }
    public MacroCall
	(SyntaxExpander macro, FragmentList modifiers, 
	 IdentifierFragment id, FragmentList fragments) {
	this.macro     = macro;
	this.modifiers = modifiers;
	this.id        = id;
	this.fragments = fragments;
	LooseFragmentParser.debugOut("NEW ARGS", fragments);
    }
    public FragmentList getInsideFragments() { return fragments; }
    public void setInsideFragments(FragmentList fs) { fragments = fs; }

    public Fragment expandTemplates () {
	getInsideFragments().expandTemplates();
	return this;
    } 
    public Fragment expandMacros (MacroExpander expander, boolean isRecursive) {
	SequenceFragment args = new SequenceFragment(fragments);
	LooseFragmentParser.trace("Invoking expander method " + expander);
	Expansion exp;
	try {
	    LooseFragmentParser.debugOut("ARGS", args.getInsideFragments());
	    exp = macro.expand(args);
	    exp.setModifiers(modifiers);
	    LooseFragmentParser.debugOut("EXPANSION", exp.getInsideFragments());
	} catch (Exception e) {
	    LooseFragmentParser.trace("Macro expansion failure " + this + " error " + e);
	    e.printStackTrace();   // so we can get stack trace		
	    exp = new Expansion(fragments);
	    return exp;
	}
	exp.markMacros(expander);
	if (isRecursive) 
	    return exp.expandMacros(expander, isRecursive);
	else
	    return exp;
    } 
    public List tokens () {
	return List.nil.push(makeToken(JavaTokenTypes.MACROCALL, ""));
    } 
}
