// Copyright 2001, Jonathan Bachrach.  See file TERMS.

import java.io.*;
import java.util.*;
import antlr.*;

/**
 * A nested fragment representing template code evaluation 
 * syntax ?(expression) 
 * @author  Jonathan Bachrach
 * @author  Keith Playford
 * @version 0.9, 03OCT00
 */
public class QuestionExpressionFragment extends NestedFragment {
    Token questionToken = null;
    private void init () {
	left  = "?(";
	right = ")";
	templateGeneratorName = "processQuestionExpression";
    }
    public QuestionExpressionFragment(FragmentList children) {
	super(children); init();
	questionToken  = makeToken(JavaTokenTypes.QUESTION, "?");
	leftToken      = makeToken(JavaTokenTypes.LPAREN, "(");
	rightToken     = makeToken(JavaTokenTypes.RPAREN, ")");
    }
    public QuestionExpressionFragment
	(Token question, Token left, FragmentList children, Token right) {
	super(children); init();
	questionToken  = left;
	leftToken  = left;
	rightToken = right;
    }
    public List tokens () {
	return List.list(questionToken)
	    .append(List.list(leftToken))
	    .append(getInsideFragments().tokens())
	    .append(List.list(rightToken));
    }
    public Expansion compilePatternElement 
	(FragmentList tail, IdentifierFragment nowVar, IdentifierFragment nextVar) {
	System.err.println("Error can't have question expression fragment in pattern " + this);
	return new Expansion(FragmentList.fnil);
    }
    public Fragment compileTemplate(int depth) {
	return compileQuestionExpression(depth, 0);
	/*
	System.out.println("?() Depth = " + depth);
	if (depth > 0)
	    return super.compileTemplate(depth - 1);
	else
	    return new Expansion(getInsideFragments());
	*/
    }
    static Fragment regenerateQuestionExpression (Fragment guts, int qDepth) {
	if (qDepth == 0)
	    return guts;
	else
	    return Template.processTemplate
		(FragmentList.fnil
		 .fpush(Template.substituteIdentifier
			("Template.processQuestionExpression"))
		 .fpush(Template.processParens
			(FragmentList.fnil.
			 fpush(regenerateQuestionExpression(guts, qDepth - 1))))
		 .freverse());
    }
    public Fragment compileQuestionExpression (int tDepth, int qDepth) {
	FragmentList guts  = getInsideFragments();
	if (tDepth == qDepth)
	    return new Expansion(guts);
	else if (guts.size() == 1) {
	    Fragment elt = guts.fhead();
	    if ((elt instanceof PatternVariableFragment) ||
		(elt instanceof QuestionExpressionFragment))
		return elt.compileQuestionExpression(tDepth, qDepth + 1);
	    else
		return regenerateQuestionExpression
		    (guts.compileTemplate(tDepth), qDepth);
	} else
	    return regenerateQuestionExpression
		(guts.compileTemplate(tDepth), qDepth);
    }
}

