// Copyright 2001, Jonathan Bachrach.  See file TERMS.

import java.io.*;
import java.util.*;
import antlr.*;

/**
 * A #{ } nested fragment used for code quotes
 * @author  Jonathan Bachrach
 * @author  Keith Playford
 * @version 0.9, 03OCT00
 */
public class Template extends NestedFragment {
    private void init () {
	left  = "#{";
	right = "}";
	templateGeneratorName = "processNestedTemplate";
    }
    public Template(FragmentList fragments) {
	super(fragments); init();
	leftToken  = makeToken(JavaTokenTypes.HASHLCURLY, "#{");
	rightToken = makeToken(JavaTokenTypes.RCURLY, "}");
    }
    public Template (Token left, FragmentList fragments, Token right) {
	super(fragments); init();
	leftToken  = left;
	rightToken = right;
    }
    public List tokens () {
	return List.list(makeToken(JavaTokenTypes.CODE_QUOTE, "#{}"));
    }
	
    public Fragment expandTemplates () { 
	setCurrentLine(getLine());
	return compileNestedTemplate(0, "processTemplate");
	// return compileTemplate(1);
    } 
    public Fragment compileTemplate(int depth) {
	System.out.println("#{} Depth = " + depth);
	return super.compileTemplate(depth + 1);
    }
    public Fragment compileQuestionExpression (int tDepth, int qDepth) {
	// TODO: is this dubious?
	return super.compileQuestionExpression(tDepth + 1, qDepth);
    }

    public Expansion compilePattern (Fragment expression, IdentifierFragment endVar) {
	FragmentList vars    = collectBoundVariables();
	Enumeration elts     = vars.elements();
	FragmentList decls   = FragmentList.fnil;
	IdentifierFragment e = IdentifierFragment.genSym("e");
	while (elts.hasMoreElements()) {
	    PatternVariableFragment var 
		= (PatternVariableFragment)elts.nextElement();
	    decls = decls.fpush(Template.substituteIdentifier
				(var.getFragmentClassName()))
		.fpush(Template.substituteIdentifier
		       (var.getVariableName().getName()))
		.fpush(Template.substituteIdentifier("="))
		.fpush(Template.substituteIdentifier("null"))
		.fpush(Template.substituteSemicolon());
	}
	return Template.processTemplate
	    (FragmentList.fnil
	     .fpush(Template.substituteIdentifier("FragmentList"))
	     .fpush(e)
	     .fpush(Template.substituteIdentifier("="))
	     .fpush(expression)
	     .fpush(Template.substituteDot())
	     .fpush(Template.substituteIdentifier("getInsideFragments"))
	     .fpush(Template.processParens(FragmentList.fnil))
	     .fpush(Template.substituteSemicolon())
	     .fpush(new Expansion(decls.freverse()))
	     .fpush(getInsideFragments().compilePatternElements(e, endVar))
	     .freverse());
    }
    public Fragment markMacros (MacroExpander expander) {
	return this;
    }
    public Fragment expandMacros (MacroExpander expander, boolean isRecursive) {
	return this;
    }
    // TODO: STUBS FOR THE REAL WORK
    public static Expansion processTemplate(FragmentList fragments) {
	return new Expansion(fragments);
    }
    public static Template processNestedTemplate(FragmentList fragments) {
	return new Template(fragments);
    }
    public static IdentifierFragment substituteIdentifier(String name) {
	return new IdentifierFragment(name);
    }
    public static PatternVariableFragment substitutePatternVariable(String name) {
	System.out.println("SPV " + name);
	return new PatternVariableFragment(name);
    }
    public static DotFragment substituteDot() {
	return new DotFragment();
    }
    public static CommaFragment substituteComma() {
	return new CommaFragment();
    }
    public static SemicolonFragment substituteSemicolon() {
	return new SemicolonFragment();
    }
    public static QuestionFragment substituteQuestion() {
	return new QuestionFragment();
    }
    public static ParensFragment processParens(FragmentList fragments) {
	return new ParensFragment(fragments);
    }
    public static BracesFragment processBraces(FragmentList fragments) {
	return new BracesFragment(fragments);
    }
    public static BracketsFragment processBrackets(FragmentList fragments) {
	return new BracketsFragment(fragments);
    }
    public static QuestionExpressionFragment processQuestionExpression
	  (FragmentList fragments) {
	return new QuestionExpressionFragment(fragments);
    }
    public static QuestionExpressionFragment processQuestionExpression
	  (Fragment fragment) {
	return new QuestionExpressionFragment
	    (FragmentList.fnil.fpush(fragment));
    }
    // public String stringify() { }
    public static StringFragment substituteAsString(String string) { 
	return new StringFragment(string);
    }
    public static IntegerFragment substituteAsInteger(Integer x) { 
	return new IntegerFragment(x);
    }
    public static IntegerFragment substituteAsInteger(int x) { 
	return substituteAsInteger(new Integer(x));
    }
    public static FloatFragment substituteAsFloat(Float x) { 
	return new FloatFragment(x);
    }
    public static FloatFragment substituteAsFloat(float x) { 
	return substituteAsFloat(new Float(x));
    }
    // public SymbolFragment substituteAsSymbol() { }
    // public FragmentList substituteSequence(
}
